/*
 * dv_utils.c
 *
 * 	Greated for:
 *	IMPROVEMENT: Excel PowerPivot is unable to show the list of tables from PostgresODBC
 *	ISSUE: REPIN-28
 *	SOLUTION: show the list of tables
 *	CREATED BY: Kopychenko Pavel
 *
 *
 *  Created on: 04.03.2013
 *      Author: Pavel kopychenko
 */

#include "dv_utils.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


/* Predicat for compare started pisition and fragmet */
int cmpSubStr(int i, const char* str, int len , const char* subStr)
{
	int si = 0;

	for(; subStr[si] != 0; ++si, ++i )
		if((i + 1) > len || subStr[si] != str[i])
			return 0;

	return 1;
}

/* Roling strig to left */
void rolStr(char* str, int len, int i, int c)
{
	for(;i < len; ++i)
		str[i] =  (i + c + 1) > len ? ' ' : str[i + c];
}

/* Replace in string pisition */
int replaceStr(char* str, int len, int i, char* subStr, int l)
{
	int si = 0;
	int subStrLen = strlen(subStr);

	for(;si < subStrLen; ++si, ++i)
		str[i] = subStr[si];

	if(l > subStrLen)
		rolStr(str, len,  i, l - subStrLen);

	return subStrLen;
}


// IMPROVEMENT: 64bit ODBC driver problem with ::timestamp and similar constructs
// ISSUE: REPIN-61
// SOLUTION: Words reordering
// FIXED BY: Kopychenko Pavel
// DATE: 27.01.2014

const char* pgtypes[] = {
				"::abstime",
				"::anyarray",
				"::anynonarray",
				"::any",
				"::bit",
				"::bool",
				"::bytea",
				"::char",
				"::cid",
				"::cstring",
				"::date",
				"::float4",
				"::float8",
				"::int2vector",
				"::interval",
				"::int2",
				"::int4",
				"::int8",
				"::line",
				"::lseg",
				"::name",
				"::numeric",
				"::oidvector",
				"::oid",
				"::path",
				"::point",
				"::record",
				"::refcursor",
				"::regproc",
				"::reltime",
				"::smgr",
				"::text",
				"::tid",
				"::timestamptz",
				"::timestamp",
				"::timetz",
				"::time",
				"::tinterval",
				"::unknown",
				"::uuid",
				"::varbit",
				"::varchar",
				"::void",
				"::xid",
				"::xml",
				0
};
// END

/* Replace query by replacement rules */
void replaceAllQueryForDV(char* str)
{
	int len = strlen(str), i = 0, j=0;

	for(; i < len; i++)
	{
		if(cmpSubStr(i, str, len , "[[["))
			i += replaceStr(str, len, i , "\"[", strlen("[[["));

		if(cmpSubStr(i, str, len , "]]]"))
			i += replaceStr(str, len, i , "]\"", strlen("]]]"));

		if(cmpSubStr(i, str, len , "[["))
			i += replaceStr(str, len, i , "[", strlen("[["));

		if(cmpSubStr(i, str, len , "]]"))
			i += replaceStr(str, len, i , "]", strlen("]]"));

		if(cmpSubStr(i, str, len , "["))
			i += replaceStr(str, len, i , "\"", strlen("["));

		if(cmpSubStr(i, str, len , "]"))
			i += replaceStr(str, len, i , "\"", strlen("]"));

		j = 0;
		while(pgtypes[j] != 0 ){
			if(cmpSubStr(i, str, len , pgtypes[j]))
				i += replaceStr(str, len, i , " ", strlen(pgtypes[j]));
			j++;
		}
	}
}

void clearBackSlash(char* str)
{
	int len = strlen(str), i = 0;

	for(; i < len; i++)
	{
		if(cmpSubStr(i, str, len , "\\"))
			i += replaceStr(str, len, i , "", strlen("\\"));
	}

	--i;
	for(;i > 0; --i)
		if(str[i] == ' ')
			str[i] = 0;
		else
			break;

}

// -----------------------------------------------------------------------------------------

void setPbuff(char* pbuff[][3], int i, char* type, char* ptr , char* value) {
	pbuff[i][0] = type;
	pbuff[i][1] = ptr;
	pbuff[i][2] = value;
}

char* getBbuffType(char* pbuff[][3], int i){
	return pbuff[i][0];
}

char* getBbuffPtr(char* pbuff[][3], int i){
	return pbuff[i][1];
}

void cleanPbuff(char* pbuff[][3]) {
	int i = 0;
	for (; i < 100; i++)
		setPbuff(pbuff, i,0,0,0);
}

void cleanQuery(char * query , int len)
{
	int i=0,j=0,s=0;

	for (i = 0; i < len; i++)
		if (query[i] == ' ' || query[i] == '\n' || query[i] == '\t') {
			++s;
			if (s == 1) {
				query[j] = ' ';
				++j;
			}
		} else {
			query[j] = query[i];
			++j;
			s = 0;
		}

	query[j - 1] = query[j - 1] == ' ' ? 0 : query[j - 1];

	for (; j < len; j++)
		query[j] = 0;
}


void splitQuery(char * query, char* pbuff[][3]) {
	char* markers[] = {MARK_BASE,MARK_SCHEME,MARK_TABLE};
	int i = 0, j = 0, x = 0, sb = 0, queryLen = strlen(query);
	for (i = 0; i < queryLen; i++) {
		for (j = 0; j < MARKS_COUNT; j++) {
			if (strncmp(&query[i], markers[j], strlen(markers[j])) == 0) {
				setPbuff(pbuff,x++,POINT_TYPE_PAR, markers[j],0);
				query[i] = 0;
				sb = 0;
				i += strlen(markers[j]);
				break;
			}
		}

		if(sb == 0)
		{
			setPbuff(pbuff,x++,POINT_TYPE_STR, &query[i],0);
			++sb;
		}
	}
}

int cmpQuerys(char* query, char* pbuff[][3]){
	int i = 0, c = 0;
	char *qpStart = query;

	for (i = 0; i < 100; i++) {
		if (pbuff[i][0] != 0 && strcmp(pbuff[i][0], POINT_TYPE_STR) == 0) {
			qpStart = (char*) strstr(qpStart, pbuff[i][1]);

			if (qpStart != 0) {
				pbuff[i][2] = qpStart;
				qpStart += strlen(pbuff[i][1]);
			} else
				return 0;
		} else
			continue;
	}

	if(strlen(qpStart) > 0)
		return 0;

	return 1;
}

void parseQuery(char* query, char* pbuff[][3]){
	int i = 0, c = 0;
	char *qpStart = query;

	for (i = 0; i < 100; i++) {
		//pbuff[i][2] = 0;

		if(pbuff[i][0] != 0 && strcmp(pbuff[i][0], POINT_TYPE_PAR) == 0){
			pbuff[i][2] =
					(i - 1) >= 0 ?
							(char*) strstr(qpStart, pbuff[i - 1][1])
									+ strlen(pbuff[i - 1][1]) :
							query;

			if(pbuff[i+1][0] >= 0)
			{
				char* ts = ((char*) strstr(qpStart, pbuff[i+1][1])) - 1;
				qpStart = ts;
			}
		}
	}
}

char* addPar(char* str, char* pbuff[][3], char* par){
	int i = 0;
	char t = 0, *result = 0;



	for (i = 0; i < 100; i++) {
		if(pbuff[i][0] != 0 && strcmp(pbuff[i][0], POINT_TYPE_PAR) == 0
				&& strcmp(pbuff[i][1], par) == 0){
			break;
		}
	}

	t = pbuff[i+1][0] > 0 ? *pbuff[i+1][2] : 0;

	if(t != 0)
		*pbuff[i+1][2] = 0;

	result = ((char*) strcpy(str, pbuff[i][2])) + strlen(pbuff[i][2]);

	if(t != 0)
		*pbuff[i+1][2] = t;

	return result;
}

void cleanStr(char* str, int len){
	int i = 0;
	for(;i < len; i++)
		str[i] = 0;
}

void createNewQuery(char* newQuery, char* sQ[] ,char* qpbuff[][3]){
	char* pNq = 0;
	int i = 0;
	char c = sQ[i][0], *line = sQ[i];

	while(c != 'e')	{
		line += 2;

		//printf("%c -> %s\n",c, line);

		if(c == 's'){
			strcpy(newQuery, line);
			newQuery += strlen(line);
		} else if(c == 'p'){
			newQuery = addPar(newQuery, qpbuff, line);
		}

		line = sQ[++i];
		c = *line;
	}
}
















