/*
 * memory_manager.c
 *
 *  Created on: 23.01.2013
 *      Author: pk
 *
 *   IMPROVEMENT: Since reading large datasets is essential for our product, please look at ways to speed up the driver.
 *   ISSUE: REPIN-11
 *   SOLUTION: New mamory management
 *   FIXED BY: Kopychenko Pavel
 *   DATE: 22.01.2013
 *
 *
 */

#include <stdlib.h>
#include <stdio.h>
//#include <windows.h>

#include "memory_manager.h"

struct MemoryBlock_* pHeadOfMemoryBlocks = NULL;
struct MemoryBlock_* pEndOfMemoryBlocks = NULL;


int min_mem_block_size = MIN_MEM_BLOCK_SIZE;
int predict_mem_cell_count = PREDICT_MEM_CELL_COUNT;
int block_count_for_gc_start = BLOCK_COUNT_FOR_GC_START;



/* is block is free */
//short int isBlockFree(MemoryBlock* block, int size) {
//	return block && !block->isCleaningMode
//			&& ((block->pLast + size) <= block->pDumpEnd);
//}

#define IS_BLOCK_FREE(block,size)\
block && !block->isCleaningMode && ((block->pLast + size) <= block->pDumpEnd)


/* Create the new memory block */
MemoryBlock* getMemoryBlock(int size) {
	MemoryBlock* mb = NULL;

	// DEBUG MESSAGE
	//printf("%d\n",min_mem_block_size);
	//printf("%d\n",predict_mem_cell_count);
	//printf("%d\n",block_count_for_gc_start);

	//-----------------------------------------------------------------------------
	if (IS_BLOCK_FREE(pEndOfMemoryBlocks, size)) {

		mb = pEndOfMemoryBlocks;
	} else if (IS_BLOCK_FREE(pHeadOfMemoryBlocks, size)) {

		mb = pHeadOfMemoryBlocks;
		pHeadOfMemoryBlocks = pHeadOfMemoryBlocks->pNextMemoryBlock;
		pEndOfMemoryBlocks->pNextMemoryBlock = mb;
		pEndOfMemoryBlocks = mb;
		pEndOfMemoryBlocks->pNextMemoryBlock = NULL;
	} else {

		// If not found free old block then
		// Create the new block
		//

		int mod =
				size <= min_mem_block_size ?
						min_mem_block_size : size * predict_mem_cell_count;

		mb = (struct MemoryBlock_*) malloc(
				sizeof(struct MemoryBlock_) + mod * sizeof(char));
		if (!mb) // if mb is NULL
		{
			// Error message!!!
//			MessageBox(NULL,"Message in memory_manager.c.\n "
//A					"Function getMemoryBlock is stoped. malloc return NULL!","ERROR!!!",MB_OK|MB_ICONERROR);

			return mb;// OLEG: NULL ptr - SEG FAULT
		}

		// initialise new block
		mb->usedFragmensCount = 0;
		mb->isCleaningMode = FALSE;
		mb->pDump = (char*) (mb + 1);
		mb->pLast = mb->pDump;
		mb->pDumpEnd = mb->pDump + mod;
		//*mb->pDumpEnd = '\0';
		//*(mb->pDumpEnd - 1) = '1';

		mb->pNextMemoryBlock = NULL;

		if (!pHeadOfMemoryBlocks) // if pHeadOfMemoryBlocks is NULL
			pHeadOfMemoryBlocks = mb;

		if (pEndOfMemoryBlocks)
			pEndOfMemoryBlocks->pNextMemoryBlock = mb;

		pEndOfMemoryBlocks = mb;

	}
	// ----------------------------------------------------------------------------

	return mb;
}

/* get memory */
char* getMemory(int size) {
	MemoryBlock* mb = getMemoryBlock(size);

	char* mem = mb->pLast;
	mb->pLast += size;
	++mb->usedFragmensCount;

	// DEBUG MESSAGE
	//MessageBox(NULL,(void*)pHeadOfMemoryBlocks,"getMemory",MB_OK|MB_ICONINFORMATION);

	return mem;
}

/* The count of clear blocks between head and end blocks */
unsigned int garbageCount() {
	unsigned int result = 0;

	MemoryBlock* mb = pHeadOfMemoryBlocks->pNextMemoryBlock;
	while (mb != pEndOfMemoryBlocks && mb != NULL ) {
		if (!mb->isCleaningMode)
			++result;

		mb = mb->pNextMemoryBlock;
	}

	return result;
}

/* garbage collector */
void gc(MemoryBlock* _pHeadOfMemoryBlocks, MemoryBlock* _pEndOfMemoryBlocks) {

	MemoryBlock* lastMb = _pHeadOfMemoryBlocks;
	MemoryBlock* mb = _pHeadOfMemoryBlocks->pNextMemoryBlock;

	while (mb != _pEndOfMemoryBlocks) {
		if (!mb->isCleaningMode && mb->usedFragmensCount == 0) {
			lastMb->pNextMemoryBlock = mb->pNextMemoryBlock;

			// DEBUG MESSAGE
			//MessageBox(NULL,"1","1",MB_OK|MB_ICONINFORMATION);

			free(mb);

			// DEBUG MESSAGE
			//MessageBox(NULL,"2","2",MB_OK|MB_ICONINFORMATION);

			mb = lastMb->pNextMemoryBlock;
		} else {
			lastMb = mb;
			mb = mb->pNextMemoryBlock;
		}
	}
}

/* free memory */
bool freeMemory(char* mem, MemoryBlock* _pHeadOfMemoryBlocks,
		MemoryBlock* _pEndOfMemoryBlocks) {

	static int gsr = 0;

	bool res = FALSE;

	MemoryBlock* mb = _pHeadOfMemoryBlocks;

	// DEBUG MESSAGE
	//MessageBox(NULL,(void*)pHeadOfMemoryBlocks,"freeMemory",MB_OK|MB_ICONINFORMATION);

	while (mb) {



		// DEBUG MESSAGE
		//MessageBox(NULL,mb->usedFragmensCount,"freeMemory *111",MB_OK|MB_ICONINFORMATION);

		if (mem >= mb->pDump && mem <= mb->pDumpEnd) {

			--mb->usedFragmensCount;
			mb->isCleaningMode = mb->usedFragmensCount > 0;

			if (mb->isCleaningMode)
				mb->pLast = mb->pDump;

			res = TRUE;

			break;
		}

		mb = mb->pNextMemoryBlock;
	}

	gsr++;

	if (gsr > GCR_MAX) {

		// DEBUG
		// printf("<%d -", garbageCount());

		if (garbageCount() > block_count_for_gc_start)
			gc(_pHeadOfMemoryBlocks, _pEndOfMemoryBlocks);

		// DEBUG
		//printf(" %d>", garbageCount());

		gsr = 0;
	}

	return res;
}

